<?php
/**
 * Класс для работы с настройками
 * 
 * @version  1.2.4
 */
class Core_Settings {
	
	/**
	 * Где хранятся настройки - в базе или в файле
	 * 
	 * db - в базе
	 * file - в файле log/settings.txt
	 * 
	 * @var string
	 */
	public $store = 'db';
	
	/**
	 * Загружаем текущие настройки
	 */
	public function __construct() {
		
		// получаем настройки из хранилища
		if ( $this->store=='db' ) {
			global $database;
			
			$rows = ( $database ) ? $database->select("SELECT * FROM ?_settings") : array();
			
			$settings = array();
			foreach ($rows as $row) {
				if ( isset($settings[ $row['name'] ]) ) continue;
				$settings[ $row['name'] ] = $row['value'];
			}
		} else {
			$settings = get_items_from_store( LOG.'/settings.txt' );
		}
		
		// прописываем настройки в поля объекта
		foreach ($settings as $name => $value) {
			if ( $this->is_serialized($value) ) $value = unserialize($value);
			$this->$name = $value;
		}
	}
	
	
	//*****************//
	// ОБЩИЕ НАСТРОЙКИ //
	//*****************//
	/**
	 * Сохранить параметр в настройках
	 * 
	 * @param string $name - название параметр
	 * @param string $value - новое значение параметра
	 */
	 public function save_option($name, $value) {
		
		if ( $this->store=='db' ) {
			save_settings($name, $value);
		} else {
			update_item_in_store(LOG.'/settings.txt', $name, $value);
		}
		
		$this->$name = $value;
		
	}
	
	/**
	 * Удалить параметр из настроек
	 *
	 * @param string $name - название параметра
	 */
	public function delete_option($name) {
		delete_settings($name);
		unset($this->$name);
	}
	
	/**
	 * Добавить значение в настройку, которая представлена в виде массива
	 *
	 * @param string $name - название параметра
	 * @param string $value - значение, которое надо добавить в массив
	 */
	public function add_value_to_option($name, $value) {
		
		# получаем настройку из базы
		$option = get_settings($name, array());
		
		# ищем значение, которое нужно добавить, в массиве
		$exists = false;
		foreach ($option as $val) {
			if ( $val != $value ) continue;
			$exists = true;
			break;
		}
		
		# если значения в массиве ещё нет - добавим и сохраним
		if ( !$exists ) {
			$option[] = $value;
			save_settings($name, serialize($option));
		}
		
		return true;
	}
	
	/**
	 * Удалить значение из настройки, которая представлена в виде массива
	 *
	 * @param string $name - название параметра
	 * @param string $value - значение, которое надо удалить из массива
	 */
	public function delete_value_from_option($name, $value) {
		
		# получаем настройку из базы
		$option = get_settings($name, array());
		
		# обходим массив и удаляем из него переданное значение
		$exists = false;
		foreach ($option as $k => $val) {
			if ( $val==$value ) {
				unset($option[$k]);
				$exists = true;
			}
		}
		
		# если настройка изменилась - сохраним
		if ( $exists ) save_settings($name, serialize($option));
	}
	
	
	//******************************//
	// НАСТРОЙКИ ОТДЕЛЬНОГО ЗАДАНИЯ //
	//******************************//
	public function save_task_option($task, $name, $value) {
		if ( !$name ) return false;
		$option_name = 'option_' . $task . '_' . $name;
		$this->save_option($option_name, $value);
	}
	
	public function add_value_to_task_option($task, $name, $value) {
		if ( !$name ) return false;
		$option_name = 'option_' . $task . '_' . $name;
		$this->add_value_to_option($option_name, $value);
	}
	
	public function delete_value_from_task_option($task, $name, $value) {
		if ( !$name ) return false;
		$option_name = 'option_' . $task . '_' . $name;
		$this->delete_value_from_option($option_name, $value);
	}
	
	public function get_task_option($task, $name, $default='') {
		if ( !$name ) return false;
		
		$option_name = 'option_' . $task . '_' . $name;
		
		if ( $this->store=='db' ) {
			global $database;
			$setting = $database->selectRow("SELECT * FROM ?_settings WHERE `name` = ?", $option_name);
			$value = ( $setting ) ? $setting['value'] : $default;
		} else {
			$settings = get_items_from_store( LOG.'/settings.txt' );
			$value = ( isset($option_name) ) ? $settings[ $option_name ] : $default;
		}
		
		return $value;
	}
	
	public function task_options($task) {
		
		$prefix = 'option_' . $task . '_';
		$settings = new stdClass();
		
		$options = get_object_vars($this);
		foreach ($options as $name => $value) {
			if ( strpos($name, $prefix)===false ) continue;
			$name = str_replace($prefix, '', $name);
			if ( !$name ) continue;
			
			$settings->$name = $value;
		}
		
		return $settings;
	}
	
	public function reset_task($task) {
		
		$prefix = 'option_' . $task . '_';
		$settings = new stdClass();
		
		$options = get_object_vars($this);
		foreach ($options as $name => $value) {
			if ( strpos($name, $prefix)===false ) continue;
			$this->delete_option($name);
		}
		
		return true;
	}
	
	
	//*************************//
	// ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ //
	//*************************//
	public function is_serialized( $data ) {
		// if it isn't a string, it isn't serialized
		if ( ! is_string( $data ) )
			return false;
		$data = trim( $data );
		if ( 'N;' == $data )
			return true;
		$length = strlen( $data );
		if ( $length < 4 )
			return false;
		if ( ':' !== $data[1] )
			return false;
		$lastc = $data[$length-1];
		if ( ';' !== $lastc && '}' !== $lastc )
			return false;
		$token = $data[0];
		switch ( $token ) {
			case 's' :
				if ( '"' !== $data[$length-2] )
					return false;
			case 'a' :
			case 'O' :
				return (bool) preg_match( "/^{$token}:[0-9]+:/s", $data );
			case 'b' :
			case 'i' :
			case 'd' :
				return (bool) preg_match( "/^{$token}:[0-9.E-]+;\$/", $data );
		}
		return false;
	}
	
}
